unit Main;

interface

uses
  Winapi.Windows, Winapi.Messages,

  System.SysUtils, System.Variants, System.Classes, System.DateUtils,

  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls,
  Vcl.ExtCtrls, Vcl.ComCtrls,

  System.TypInfo,
  System.NetEncoding,

  //Data.DBXJSONReflect,
  REST.JSONReflect,

  Execute.JSON;

type
  TForm1 = class(TForm)
    TabControl1: TTabControl;
    Memo1: TMemo;
    Splitter1: TSplitter;
    RichEdit1: TRichEdit;
    procedure FormCreate(Sender: TObject);
    procedure TabControl1Change(Sender: TObject);
  private
    { Dclarations prives }
    FSource: string;
    procedure Source(const Name: string);
    procedure Highlight(const Keyword, Text: string);
    procedure HighlightStrings(const Text: string);
    procedure HighlightComments(const Text: string);
    procedure Marshal(Obj: TObject; const Name: string; Marshal: TJSONMarshal = nil);
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//<DEMO1>
//
// Execute.JSON use TypeInfo to convert any Type to a JSON string or populate that string to a variable
//
//  Just add a Uses to Execute.JSON and go !
//
type
  TMyValue = (Value1, Value2, Value3);
  TMySet = set of TMyValue;

  TGeneralObject = class
    ss: ShortString;
    ac: AnsiChar;
    uc: Char;
    bt: Byte;
    sm: SmallInt;
    sh: ShortInt;
    wd: Word;
    it: Integer;
    cd: Cardinal;
    wi: Int64;
    ui: UInt64;
    st: string;
    sa: AnsiString;
    ti: TTime;
    da: TDate;
    dt: TDateTime;
    mv: TMyValue;
    ms: TMySet;
  end;

procedure Demo1;
var
  o: TGeneralObject;
  s: string;
  i: Integer;
  d: TDateTime;
begin
  Form1.Memo1.Text := 'Simple demo';
  Form1.Memo1.Lines.Add('');

  o := TGeneralObject.Create;
  o.ss := '->ShortString';
  o.ac := 'a';
  o.uc := 'w';
  o.bt := 1;
  o.sm := -2;
  o.sh := +2;
  o.wd := 5;
  o.it := -5;
  o.cd := +6;
  o.wi := -7;
  o.ui := +7;
  o.st := '->string';
  o.sa := '->AnsiString';
  o.ti := Now();
  o.da := Now();
  o.dt := Now();
  o.mv := Value2;
  o.ms := [Value1, Value3];

  Form1.Memo1.Lines.Add('o.toJSON()');
  s := o.toJSON(); // Execute.JSON in action !
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('');

  Form1.Marshal(o, 'o');

  Form1.Memo1.Lines.Add('o.fromJSON()');
  o := TGeneralObject.Create;
  o.fromJSON(s);   // Execute.JSON in action !
  Form1.Memo1.Lines.Add('ss = ' + o.ss);
  Form1.Memo1.Lines.Add('ac = ' + o.ac);
  Form1.Memo1.Lines.Add('uc = ' + o.uc);
  Form1.Memo1.Lines.Add('bt = ' + IntToStr(o.bt));
  Form1.Memo1.Lines.Add('sm = ' + IntToStr(o.sm));
  Form1.Memo1.Lines.Add('sh = ' + IntToStr(o.sh));
  Form1.Memo1.Lines.Add('wd = ' + IntToStr(o.wd));
  Form1.Memo1.Lines.Add('it = ' + IntToStr(o.it));
  Form1.Memo1.Lines.Add('cd = ' + IntToStr(o.cd));
  Form1.Memo1.Lines.Add('wi = ' + IntToStr(o.wi));
  Form1.Memo1.Lines.Add('ui = ' + IntToStr(o.ui));
  Form1.Memo1.Lines.Add('st = ' + o.st);
  Form1.Memo1.Lines.Add('sa = ' + o.sa);
  Form1.Memo1.Lines.Add('ti = ' + TimeToStr(o.ti));
  Form1.Memo1.Lines.Add('da = ' + DateToStr(o.da));
  Form1.Memo1.Lines.Add('dt = ' + DateTimeToStr(o.dt));
  if o.mv = Value2 then
    Form1.Memo1.Lines.Add('mv = Value2');
  if o.ms = [Value1, Value3] then
    Form1.Memo1.Lines.Add('ff = [Value1, Value3]');
  o.Free;
  Form1.Memo1.Lines.Add('');

// Syntax in vesrion 1
//  Form1.Memo1.Lines.Add(toJSON(@s, TypeInfo(string)));

// Syntax in version 2
//  Form1.Memo1.Lines.Add(JSON<string>.toJSON(s));

// Syntax in version 3
  Form1.Memo1.Lines.Add('JSON.toJSON(s)');
  s := 'String is also a valid "Type" for Execute.JSON';
  Form1.Memo1.Lines.Add(JSON.toJSON(s));
  Form1.Memo1.Lines.Add('');

  JSON.fromJSON(s, '"JSON \"string\""');
  Form1.Memo1.Lines.Add('JSON.fromJSON(''"JSON \"string\""'')');
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('');

  Form1.Memo1.Lines.Add('s := JSON.toJSON(123)');
  s := JSON.toJSON(123);
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('JSON.fromJSON(i, s)');
  i := 0;
  JSON.fromJSON(i, s);
  Form1.Memo1.Lines.Add('i = ' + IntToStr(i));
  Form1.Memo1.Lines.Add('');

  Form1.Memo1.Lines.Add('s := JSON.toJSON(Now())');
  s := JSON.toJSON(Now());
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('JSON.fromJSON(d, s)');
  d := 0;
  JSON.fromJSON(d, s);
  Form1.Memo1.Lines.Add('d = ' + DateTimeToStr(d));
  Form1.Memo1.Lines.Add('');

end;
//</DEMO1>

//<DEMO2>
//
//  Execute.JSON can handle record and object, but records do not have generic helpers
//
type
  TSimpleObject = class
    Name : string;
    Date : TDateTime;
    Color: TColor;
    Alive: Boolean;
    Lines: TArray<string>;
    Bonus: string;
  end;

  TSimpleRecord = record
    Name : string;
    Date : TDateTime;
    Color: TColor;
    Alive: Boolean;
    Lines: TArray<string>;
    Malus: string;
  end;

procedure Demo2;
var
  o: TSimpleObject;
  r: TSimpleRecord;
  s: string;
begin
  Form1.Memo1.Text := 'Record demo';
  Form1.Memo1.Lines.Add('');

  o := TSimpleObject.Create;
  o.Name := 'Paul';
  o.Date := Now();
  o.Color := clRed;
  o.Alive := True;
  o.Lines := ['One', 'Two', 'Three'];
  o.Bonus := 'Lost value !';

  Form1.Memo1.Lines.Add('o.toJSON()');
  s := o.toJSON();  // Execute.JSON in action !
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('');

  Form1.Marshal(o, 'o');

  o.Free;

  Form1.Memo1.Lines.Add('JSON.fromJSON(r, s)');
  JSON.fromJSON(r, s); // Execute.JSON in action !
  Form1.Memo1.Lines.Add('Name = ' + r.Name);
  Form1.Memo1.Lines.Add('Date = ' + DateTimeToStr(r.Date));
  Form1.Memo1.Lines.Add('Color = ' + GetEnumName(TypeInfo(TColor), r.Color));
  Form1.Memo1.Lines.Add('Alive = ' + GetEnumName(TypeInfo(Boolean), Ord(r.Alive)));
  Form1.Memo1.Lines.Add('Lines = (' + r.Lines[0] + ', ' + r.Lines[1] + ', ' + r.Lines[2] + ')');
  Form1.Memo1.Lines.Add('Malus = ' + r.Malus);
  Form1.Memo1.Lines.Add('');

  Form1.Memo1.Lines.Add('JSON.toJSON(r)');
  s := JSON.toJSON(r);  // Execute.JSON in action !
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('');
end;
//</DEMO2>

//<DEMO3>
//
//  Execute.JSON search for two magic methods JSONBuildField and JSONParseField to let you define custom properties handling
//
type
  TCustomFieldsObject = class
    Name    : string;
    Data    : TBytes;
    Password: string;
    function JSONBuildField(const Name: string; Builder: TJSONBuilder): Boolean;
    function JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
  end;

function TCustomFieldsObject.JSONBuildField(const Name: string; Builder: TJSONBuilder): Boolean;
begin
  if Name = 'Password' then
  begin
    Builder.AppendString(System.NetEncoding.TNetEncoding.Base64.Encode(Password));
    Exit(True);
  end;

  Result := False;
end;

function TCustomFieldsObject.JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
begin
  if Name = 'Password' then
  begin
    Password := System.NetEncoding.TNetEncoding.Base64.Decode(Parser.GetString);
    Exit(True);
  end;

  Result := False;
end;

procedure Demo3;
var
  o: TCustomFieldsObject;
  s: string;
  m: TJSONMarshal;
begin
  Form1.Memo1.Text := 'Custom Fields demo';
  Form1.Memo1.Lines.Add('');

  o := TCustomFieldsObject.Create;
  o.Name := 'Paul';
  o.Data := [1, 2, 3];
  o.Password := 'Secret';

  Form1.Memo1.Lines.Add('o.toJSON()');
  s := o.toJSON();
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('');

  m := TJSONMarshal.Create(TJSONConverter.Create);
  m.RegisterConverter(TCustomFieldsObject, 'Password', function(Data: TObject; Field: string): string
  begin
    Result := System.NetEncoding.TNetEncoding.Base64.Encode(TCustomFieldsObject(Data).Password);
  end);
  Form1.Marshal(o, 'o', m);
  m.Free;

  o.Free;

  Form1.Memo1.Lines.Add('o.fromJSON()');
  o := TCustomFieldsObject.Create;
  o.fromJSON(s);
  Form1.Memo1.Lines.Add('Name = ' + o.Name);
  Form1.Memo1.Lines.Add('Data = [' + IntToStr(o.Data[0]) + ', ' + IntToStr(o.Data[1]) + ', ' + IntToStr(o.Data[2]) + ']');
  Form1.Memo1.Lines.Add('Password = ' + o.Password);
  o.Free;

end;
//</DEMO3>

//<DEMO4>
//
// Execute.JSON supports inheritance
//
type
  TParentObject = class
    Name    : string;
    Data    : TBytes;
    Password: string;
    function JSONBuildField(const Name: string; Builder: TJSONBuilder): Boolean; virtual;
    function JSONParseField(const Name: string; Parser: TJSONParser): Boolean; virtual;
  end;

  TChildObject = class(TParentObject)
    Value: Currency;
    function JSONBuildField(const Name: string; Builder: TJSONBuilder): Boolean; override;
    function JSONParseField(const Name: string; Parser: TJSONParser): Boolean; override;
  end;

function TParentObject.JSONBuildField(const Name: string; Builder: TJSONBuilder): Boolean;
begin
  if Name = 'Password' then
  begin
    Builder.AppendString(System.NetEncoding.TNetEncoding.Base64.Encode(Password));
    Exit(True);
  end;

  Result := False;
end;

function TParentObject.JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
begin
  if Name = 'Password' then
  begin
    Password := System.NetEncoding.TNetEncoding.Base64.Decode(Parser.GetString);
    Exit(True);
  end;

  Result := False;
end;

function TChildObject.JSONBuildField(const Name: string; Builder: TJSONBuilder): Boolean;
begin
  if Name = 'Value' then
  begin
  // use FormatSettings.CurrencyString as an object property
    Builder.Append('{');
    Builder.AppendString(FormatSettings.CurrencyString);
    Builder.Append(':');
  // use Value as the property value
    Builder.Append(CurrToStr(Value, JSONFormat));
    Builder.Append('}');
    Exit(True);
  end;

  Result := inherited JSONBuildField(Name, Builder);
end;

function TChildObject.JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
var
  Curr: string;
begin
  if Name = 'Value' then
  begin
    if Parser.BeginObject then
    begin
      Curr := Parser.GetKey;
      if Curr <> FormatSettings.CurrencyString then
        ShowMessage('Warning, not the same currencyString : ' + Curr + '/' + FormatSettings.CurrencyString);
      Value := Parser.GetCurrency;
      Parser.EndObject;
    end;
    Exit(True);
  end;

  Result := inherited JSONParseField(Name, Parser);
end;

procedure Demo4;
var
  o: TChildObject;
  s: string;
  m: TJSONMarshal;
begin
  Form1.Memo1.Text := 'Heritage demo';
  Form1.Memo1.Lines.Add('');

  o := TChildObject.Create;
  o.Name := 'Paul';
  o.Data := [1, 2, 3];
  o.Password := 'Secret';
  o.Value := 1.25;

  Form1.Memo1.Lines.Add('o.toJSON()');
  s := o.toJSON();
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('');


  m := TJSONMarshal.Create(TJSONConverter.Create);
  m.RegisterConverter(TChildObject, 'Password', function(Data: TObject; Field: string): string
  begin
    Result := System.NetEncoding.TNetEncoding.Base64.Encode(TChildObject(Data).Password);
  end);
  // don't know how to create the {"": 1.25} object ?!
  Form1.Marshal(o, 'o', m);
  m.Free;

  o.Free;

  Form1.Memo1.Lines.Add('o.fromJSON()');
  o := TChildObject.Create;
  o.fromJSON(s);
  Form1.Memo1.Lines.Add('Name = ' + o.Name);
  Form1.Memo1.Lines.Add('Data = [' + IntToStr(o.Data[0]) + ', ' + IntToStr(o.Data[1]) + ', ' + IntToStr(o.Data[2]) + ']');
  Form1.Memo1.Lines.Add('Password = ' + o.Password);
  Form1.Memo1.Lines.Add('Value = ' + CurrToStr(o.Value));
  o.Free;

end;
//</DEMO4>

//<DEMO5>
//
// Execute.JSON supports TStringList as an array of string
//
type
  TListObject = class
    Lines: TStringList;
    constructor Create;
    destructor Destroy; override;
  end;

constructor TListObject.Create;
begin
  inherited;
  Lines := TStringList.Create;
end;

destructor TListObject.Destroy;
begin
  Lines.Free;
  inherited;
end;

procedure Demo5;
var
  o: TListObject;
  t: TSimpleObject;
  s: string;
begin
  Form1.Memo1.Text := 'TStringList demo';
  Form1.Memo1.Lines.Add('');

  o := TListObject.Create;
  o.Lines.Add('Un');
  o.Lines.Add('Deux');
  o.Lines.Add('Trois');
  t := TSimpleObject.Create;
  o.Lines.AddObject('LostObject', t);  // t will be saved, but without a custom JSONParseField method it will be ignored in fromJSON()

  Form1.Memo1.Lines.Add('o.toJSON()');
  s := o.toJSON();
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('');

  Form1.Marshal(o, 'o');

  o.Free;
  t.Free;

  Form1.Memo1.Lines.Add('o.fromJSON()');
  o := TListObject.Create;
  o.fromJSON(s);
  Form1.Memo1.Lines.Add('Lines = ' + o.Lines.CommaText);
  o.Free;
end;
//</DEMO5>

//<DEMO6>
//
// JSONParseField and JSONBuildField can handle sub components properties
//
type
  TSubObject = class
    Name : string;
    Date : TDateTime;
    function JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
  end;

  TSubRecord = record
    Name : string;
    Date : TDateTime;
  end;

  TMainObject = class
    Name : string;
    Sub1 : TSubObject;
    Sub2 : TSubRecord;
    constructor Create;
    destructor Destroy; override;
    function JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
  end;


function TSubObject.JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
begin
  if Name = 'Date' then
  begin
     Date := Parser.GetDateTime;
     Exit(True);
  end;

  Result := False;
end;

function TMainObject.JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
begin
  if Name = 'Sub1.Date' then // never occurs cause TSubObject handles "Date"
  begin
     Sub1.Date := Parser.GetDateTime;
     Exit(True);
  end;

  if Name = 'Sub2.Date' then
  begin
     Sub2.Date := Parser.GetDateTime;
     Exit(True);
  end;

  Result := False;
end;

constructor TMainObject.Create;
begin
  inherited;
  Sub1 := TSubObject.Create;
end;

destructor TMainObject.Destroy;
begin
  Sub1.Free;
  inherited;
end;

procedure Demo6;
var
  o: TMainObject;
  s: string;
begin
  Form1.Memo1.Text := 'SubObject demo';
  Form1.Memo1.Lines.Add('');

  o := TMainObject.Create;
  o.Name := 'Paul';
  o.Sub1.Name := 'SubObject';
  o.Sub1.Date := Now();
  o.Sub2.Name := 'SubRecord';
  o.Sub2.Date := Now();

  Form1.Memo1.Lines.Add('o.toJSON()');
  s := o.toJSON();
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('');

  Form1.Marshal(o, 'o');

  o.Free;

  Form1.Memo1.Lines.Add('o.FromJSON');
  o := TMainObject.Create;
  o.FromJSON(s);
  Form1.Memo1.Lines.Add('Name = ' + o.Name);
  Form1.Memo1.Lines.Add('Sub1.Name = ' + o.Sub1.Name);
  Form1.Memo1.Lines.Add('Sub1.Date = ' + DateTimeToStr(o.Sub1.Date));
  Form1.Memo1.Lines.Add('Sub2.Name = ' + o.Sub2.Name);
  Form1.Memo1.Lines.Add('Sub2.Date = ' + DateTimeToStr(o.Sub2.Date));
  Form1.Memo1.Lines.Add('');
end;
//</DEMO6>

//<DEMO7>
//
// Support for CR/LF
//
type
  TMultiLine = class
    s: string;
  end;
procedure Demo7;
var
  m: TMultiLine;
  s: string;
begin
  Form1.Memo1.Text := 'MultiLine demo';
  Form1.Memo1.Lines.Add('');

  m := TMultiLine.Create;
  m.s := 'FirstLine'#13#10'SecondLine'#13'ThirdLine'#10'LastLine';

  Form1.Memo1.Lines.Add('m.toJSON()');
  s := m.toJSON();
  Form1.Memo1.Lines.Add(s);
  Form1.Memo1.Lines.Add('');

  Form1.Marshal(m, 'm');

  m.s := '';
  m.FromJSON(s);
  Form1.Memo1.Lines.Add('m.fromJSON(s)');
  Form1.Memo1.Lines.Add('m.s = ' + m.s);
  Form1.Memo1.Lines.Add('');
  Form1.Memo1.Lines.Add('Note that #10 and #13 alone are not supported by TMemo')

end;
//</DEMO7>

//<DEMO8>
//
// Execute.JSON and RTTI limits
//
type
// inline static array do not have RTTI informations
  TError1 = class
    v: array[Boolean, 0..2] of Byte;
  end;

  TByteArray = array[Boolean, 0..2] of Byte;
  TSolution1 = class
    v: TByteArray; // Note that Execute.JSON will see an array of 2 x 3 Bytes
  end;

// recursive reference
  TOwnerObject = class
    Name  : string;
    Child : TObject;
  end;

  TError2 = class
    Owner: TOwnerObject;
  end;

  TSolution2 = class
    Owner: TOwnerObject;
    function JSONBuildField(const Name: string; Builder: TJSONBuilder): Boolean;
    function JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
  end;

function TSolution2.JSONBuildField(const Name: string; Builder: TJSONBuilder): Boolean;
begin
  if Name = 'Owner' then
  begin
    if Owner = nil then
      Builder.Append(JSON_NIL)
    else
      Builder.AppendString(Owner.Name);
    Exit(True);
  end;

  Result := False;
end;

function TSolution2.JSONParseField(const Name: string; Parser: TJSONParser): Boolean;
begin
  if Name = 'Owner' then
  begin
    Parser.GetString; // need to locate the Owner by it's nae
    Exit(True);
  end;

  Result := False;
end;

procedure Demo8;
var
  e1: TError1;
  s1: TSolution1;
  oo: TOwnerObject;
  m : TJSONMarshal;
begin
  ShowMessage('Warning ! in this demo the application can crash even with a try/except because of REST.JSONReflect code...');

  Form1.Memo1.Text := 'Error cases:';
  Form1.Memo1.Lines.Add('');

  Form1.Memo1.Lines.Add('e1.toJSON()');
  e1 := TError1.Create;
  try
    Form1.Memo1.Lines.Add(e1.toJSON());
  except
    on e: EJSONError do
    begin
      Form1.Memo1.Lines.Add(e.Message);
    end;
  end;
  Form1.Memo1.Lines.Add('');

  Form1.Marshal(e1, 'e1');

  e1.Free;

  Form1.Memo1.Lines.Add('s1.toJSON()');
  s1 := TSolution1.Create;
  Form1.Memo1.Lines.Add(s1.toJSON());
  Form1.Memo1.Lines.Add('');

  Form1.Marshal(s1, 's1');

  s1.Free;


  Form1.Memo1.Lines.Add('oo.toJSON()');
  oo := TOwnerObject.Create;
  oo.Name := 'Error2';
  oo.Child := TError2.Create;
  TError2(oo.Child).Owner := oo;
  try
    Form1.Memo1.Lines.Add(oo.toJSON());
  except
    on e: EJSONError do
    begin
      Form1.Memo1.Lines.Add(e.Message);
    end;
  end;
  Form1.Memo1.Lines.Add('');

  Form1.Marshal(oo, 'oo');

  oo.Child.Free;
  oo.Free;
  Form1.Memo1.Lines.Add('');

  Form1.Memo1.Lines.Add('oo.toJSON()');
  oo := TOwnerObject.Create;
  oo.Name := 'Solution2';
  oo.Child := TSolution2.Create;
  TSolution2(oo.Child).Owner := oo;
  Form1.Memo1.Lines.Add(oo.toJSON());

  m := TJSONMarshal.Create(TJSONConverter.Create);
  m.RegisterConverter(TSolution2, 'Owner', function(Data: TObject; Field: string): string
  begin
    Result := TSolution2(Data).Owner.Name;
  end);
  Form1.Marshal(oo, 'oo', m);
  m.Free;

  oo.Child.Free;
  oo.Free;
end;

//</DEMO8>

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  Stream: TResourceStream;
  Ansi  : AnsiString;
begin
  Stream := TResourceStream.Create(hInstance, 'Source', RT_RCDATA);
  SetLength(Ansi, Stream.Size);
  Stream.Read(Ansi[1], Length(Ansi));
  Stream.Free;
  FSource := Ansi;

  TabControl1Change(Self);
end;

procedure TForm1.Highlight(const Keyword, Text: string);
var
  Index: Integer;
begin
  Index := Pos(Keyword, Text);
  while Index > 0 do
  begin
    if not CharInSet(Text[Index - 1] , ['a'..'z']) then
    begin
      RichEdit1.SelStart := Index - 1;
      RichEdit1.SelLength := Length(Keyword);
      RichEdit1.SelAttributes.Color := clNavy;
      RichEdit1.SelAttributes.Style := [fsBold];
    end;
    Index := Pos(Keyword, Text, Index + 1);
  end;
end;

procedure TForm1.HighlightComments(const Text: string);
var
  Index: Integer;
  Eol  : Integer;
begin
  Index := Pos('//', Text);
  while Index > 0 do
  begin
    Eol := Pos(#13, Text, Index);
    RichEdit1.SelStart := Index - 1;
    RichEdit1.SelLength := Eol - Index;
    RichEdit1.SelAttributes.Color := clGreen;
    RichEdit1.SelAttributes.Style := [fsItalic];
    Index := Pos('//', Text, Eol + 1);
  end;
end;

procedure TForm1.HighlightStrings(const Text: string);
var
  Index  : Integer;
  Comment: Integer;
  EoS    : Integer;
begin
  Index := Pos('''', Text);
  while Index > 0 do
  begin
    Comment := Index;
    while (Comment > 0) and (Text[Comment] <> #13) do
      Dec(Comment);
    Comment := Pos('//', Text, Comment);
    if (Comment = 0) or (Comment > Index)  then
    begin
      EoS := Pos('''', Text, Index + 1);
      if EoS < Index then
        Break;
      RichEdit1.SelStart := Index - 1;
      RichEdit1.SelLength := EoS - Index + 1;
      RichEdit1.SelAttributes.Color := clBlue;
      RichEdit1.SelAttributes.Style := []; // clear bolded "string"
    end else begin
      EoS := Pos(#13, Text, Comment + 1);
    end;
    Index := Pos('''', Text, EoS + 1);
  end;
end;

procedure TForm1.Marshal(Obj: TObject; const Name: string; Marshal: TJSONMarshal = nil);
var
  m: TJSONMarshal;
  s: string;
begin
  Memo1.Lines.Add('m.Marshal(' + Name + ').toJSON()');
  if Marshal = nil then
    m := TJSONMarshal.Create(TJSONConverter.Create)
  else
    m := Marshal;
  try
    s := m.Marshal(Obj).toJSON;
    Memo1.Lines.Add(s);
  except
    on e: Exception do
    begin
      Memo1.Lines.Add(e.Message);
    end;
  end;
  if Marshal = nil then
    m.Free;
  Memo1.Lines.Add('');
end;

procedure TForm1.Source(const Name: string);
var
  Start: Integer;
  Stop : Integer;
  Text : string;
  Index: Integer;
begin
  Start := Pos('<' + Name + '>', FSource) + 2 + Length(Name) + 2;
  Stop := Pos('//</' + Name + '>', FSource);
  Text := StringReplace(Copy(FSource, Start, Stop - Start), #13#10, #13, [rfReplaceAll]);

  RichEdit1.Lines.BeginUpdate;
  RichEdit1.Lines.Text := Text;

  Highlight('if ', Text);
  Highlight('on ', Text);
  Highlight('of ', Text);
  Highlight('end', Text);
  Highlight('var', Text);
  Highlight('try', Text);
  Highlight('then', Text);
  Highlight('else', Text);
  Highlight('type', Text);
  Highlight('const', Text);
  Highlight('begin', Text);
  Highlight('class', Text);
  Highlight('array', Text);
  Highlight('except', Text);
  Highlight('set of', Text);
  Highlight('record', Text);
  Highlight('string', Text);
  Highlight('virtual', Text);
  Highlight('function', Text);
  Highlight('override', Text);
  Highlight('procedure', Text);
  Highlight('inherited', Text);
  Highlight('destructor', Text);
  Highlight('constructor', Text);
  HighlightStrings(Text);
  HighlightComments(Text);

  RichEdit1.SelStart := 0;
  RichEdit1.SelLength := 0;
  RichEdit1.Lines.EndUpdate;
end;

procedure TForm1.TabControl1Change(Sender: TObject);
begin
  case TabControl1.TabIndex of
    0: begin
      RichEdit1.Lines.Text :=
        'Execute.JSON, JSON builder/parser for Delphi XE8 (c)2015 Execute SARL'#13 +
        'Paul TOTH <contact@execute.fr>'#13+
        'http://www.execute.fr'#13+
        #13+
        'Execute.JSON is a Delphi unit to convert Delphi types to JSON string and vice versa.'#13+
        'The unit use only RTTI from "TypeInfo".'#13+
        #13+
        'Look at the provided samples to compare Execute.JSON and REST.JSONReflect.';
      Memo1.Clear;
    end;
    1: begin
      Source('DEMO1');
      Demo1();
    end;
    2: begin
      Source('DEMO2');
      Demo2();
    end;
    3: begin
      Source('DEMO3');
      Demo3();
    end;
    4: begin
      Source('DEMO4');
      Demo4();
    end;
    5: begin
      Source('DEMO5');
      Demo5();
    end;
    6: begin
      Source('DEMO6');
      Demo6();
    end;
    7: begin
      Source('DEMO7');
      Demo7();
    end;
    8: begin
      Source('DEMO8');
      Demo8();
    end;
  end;
end;

end.
